@@ -3,7 +3,6 @@ import sys
 import time
 from PySide.QtCore import *
 from PySide.QtGui import *
-#from PySide.QtMaemo5 import * 
 
 # Some global variables
 CommentChar = '#'
@@ -228,7 +227,7 @@ class AboutWindow(QMainWindow):
         self.setAttribute(Qt.WA_Maemo5StackedWindow)
         self.setAttribute(Qt.WA_DeleteOnClose)
         self.label = QLabel(self)
-        AboutTxt = "<h3>Gngeo-gui 0.0.5-4</h3><p>" \
+        AboutTxt = "<h3>Gngeo-gui 0.0.5-5</h3><p>" \
                    "<img src=/opt/usr/share/icons/hicolor/64x64/apps/gngeo_gui.png /><br>" \
                    "GnGeo-gui is a Graphical User Interface for GnGeo<p> " \
                    "GnGeo emulates the NeoGeo gameconsole from SNK &#174;</p>" \
@@ -250,7 +249,7 @@ class ConfigWindow(QMainWindow):
 	wg = QWidget()
 	area.setWidget(wg)
 	self.setCentralWidget(area)
-	wg.resize(790,600);
+	wg.resize(790,660);
 	grid = QGridLayout(wg)
 	wg.show()
 	area.show()
@@ -324,13 +323,8 @@ class ConfigWindow(QMainWindow):
 		self.joystickPushButton.setChecked(True)
 	# system
 	grid.addWidget(QLabel("<center>system:</center>"), 4, 1)
-	#self.adebutton = QMaemo5ValueButton(self)
-        #self.adebutton.setValueText('bla bla!')
-	#self.adebutton.setText('Push me')
-	#self.adebutton.setValueLayout(QMaemo5ValueButton.ValueBesideText)
-        #grid.addWidget(self.adebutton,9,3)
         self.comboSystem = QComboBox(self)
-	self.comboSystem.setStatusTip('system')
+#	self.comboSystem.setStatusTip('system')
         grid.addWidget(self.comboSystem,5,1)
 	systems = self.listDefaultFirst("system",['arcade','home'])
 	for sysType in systems:
@@ -348,7 +342,7 @@ class ConfigWindow(QMainWindow):
         grid.addWidget(self.comboEffect,5,3)
 	effects = self.listDefaultFirst("effect",['none', 'scanline', 'scanline50', 'scale2x', 'scale3x', 'scale4x', 'scale2x50', 'scale2x75', 'hq2x', 'hq3x', 'lq2x', 'lq3x', 'doublex'])
 	for effect in effects:
-       		 	self.comboEffect.addItem(effect)
+		self.comboEffect.addItem(effect)
 	# Samplerate
 	grid.addWidget(QLabel("<center>sound rate:</center>"), 6, 1)
         self.comboSampleRate = QComboBox(self)
@@ -374,40 +368,54 @@ class ConfigWindow(QMainWindow):
 	self.M68KSpinBox.setDecimals(0)
 	self.M68KSpinBox.setSuffix(" %")
 	grid.addWidget(self.M68KSpinBox, 7,3)
+	# Joystick device P1 (/dev/input/js?)
+	grid.addWidget(QLabel("<center>P1 joystick device:</center>"), 8,1)
+        self.comboJoyp1dev = QComboBox(self)
+        grid.addWidget(self.comboJoyp1dev,9,1)
+	JoyP1devs = self.listDefaultFirst("p1joydev",['0', '1', '2', '3'])
+	for JoyP1dev in JoyP1devs:
+       		self.comboJoyp1dev.addItem(JoyP1dev)
+	# Joystick device P2 (/dev/input/js?)
+	grid.addWidget(QLabel("<center>P2 joystick device:</center>"), 8,2)
+        self.comboJoyp2dev = QComboBox(self)
+        grid.addWidget(self.comboJoyp2dev,9,2)
+	JoyP2devs = self.listDefaultFirst("p2joydev",['0', '1', '2', '3'])
+	for JoyP2dev in JoyP2devs:
+       		self.comboJoyp2dev.addItem(JoyP2dev)
+        # Exit on start button
+        self.exitPushButton = QPushButton("exit after start game",self)
+	self.exitPushButton.setCheckable(True)
+        grid.addWidget(self.exitPushButton,9,3)
+	if self.gui_opts["exit_after_launch"] == "true":
+		self.exitPushButton.setChecked(True)
 	# Path button
         self.pathButton = QPushButton("paths def",self)
         self.pathButton.setFont(font)
-        grid.addWidget(self.pathButton,8,1)
+        grid.addWidget(self.pathButton,10,1)
 	QObject.connect(self.pathButton
 		, SIGNAL("clicked()")
 		, self.on_paths_clicked)
 	# Keys button
         self.keysButton = QPushButton("key def",self)
         self.keysButton.setFont(font)
-        grid.addWidget(self.keysButton,8,2)
+        grid.addWidget(self.keysButton,10,2)
 	QObject.connect(self.keysButton
 		, SIGNAL("clicked()")
 		, self.on_keys_clicked)
 	# Joystick config
         self.joyconfButton = QPushButton("joystick def",self)
         self.joyconfButton.setFont(font)
-        grid.addWidget(self.joyconfButton,8,3)
+        grid.addWidget(self.joyconfButton,10,3)
 	QObject.connect(self.joyconfButton
 		, SIGNAL("clicked()")
 		, self.on_defjoy_clicked)
-        # Exit on start button
-        self.exitPushButton = QPushButton("exit after start game",self)
-	self.exitPushButton.setCheckable(True)
-        grid.addWidget(self.exitPushButton,9,1)
-	if self.gui_opts["exit_after_launch"] == "true":
-		self.exitPushButton.setChecked(True)
 	# Save button
         self.saveButton = QPushButton("SAVE",self)
         self.saveButton.setStyleSheet(
 	"QPushButton { color: red; font:bold; }"
       		) 
         self.saveButton.setFont(font)
-        grid.addWidget(self.saveButton,9,2)
+        grid.addWidget(self.saveButton,12,1,1,3)
 	QObject.connect(self.saveButton
 		, SIGNAL("clicked()")
 		, self.on_save_clicked)
@@ -450,6 +458,8 @@ class ConfigWindow(QMainWindow):
 	self.options["country"] = self.comboCountry.currentText()
 	self.options["effect"] = self.comboEffect.currentText()
 	self.options["samplerate"] = self.comboSampleRate.currentText()
+	self.options["p1joydev"] = self.comboJoyp1dev.currentText()
+	self.options["p2joydev"] = self.comboJoyp2dev.currentText()
 	if BiosPath != "":
 		self.options["biospath"] = BiosPath
 	if RomrcPath != "":
@@ -881,8 +891,6 @@ class JoykeyWindow(QDialog):
         super(JoykeyWindow, self).__init__(parent)
 	joystickAttached = os.path.exists("/dev/input/js0")
 	if (joystickAttached):
-		global pipe
-		pipe = open('/dev/input/jsx','r')
         	self.setWindowTitle("GnGeo-gui joystick config")
 	else:
         	self.setWindowTitle("Gngeo-gui joystick config (no USB joystick found)")
@@ -1066,26 +1074,26 @@ class JoykeyWindow(QDialog):
 	grid.addWidget(self.P2RightjoykeySpinBox, 11, 3)
 
 	if (joystickAttached):
-		self.P2AjoyButton = MyJoykeyPressed("button",self)
-		self.P1BjoyButton = MyJoykeyPressed("button",self)
-		self.P2BjoyButton = MyJoykeyPressed("button",self)
-		self.P1CjoyButton = MyJoykeyPressed("button",self)
-		self.P2CjoyButton = MyJoykeyPressed("button",self)
-		self.P1DjoyButton = MyJoykeyPressed("button",self)
-		self.P2DjoyButton = MyJoykeyPressed("button",self)
-		self.P1StartjoyButton = MyJoykeyPressed("button",self)
-		self.P2StartjoyButton = MyJoykeyPressed("button",self)
-		self.P1CoinjoyButton = MyJoykeyPressed("button",self)
-		self.P2CoinjoyButton = MyJoykeyPressed("button",self)
-		self.P1UpjoyButton = MyJoykeyPressed("button",self)
-		self.P2UpjoyButton = MyJoykeyPressed("button",self)
-		self.P1AjoyButton = MyJoykeyPressed("button",self)
-		self.P1RightjoyButton = MyJoykeyPressed("button",self)
-		self.P2RightjoyButton = MyJoykeyPressed("button",self)
-		self.P2LeftjoyButton = MyJoykeyPressed("button",self)
-		self.P1LeftjoyButton = MyJoykeyPressed("button",self)
-		self.P2DownjoyButton = MyJoykeyPressed("button",self)
-		self.P1DownjoyButton = MyJoykeyPressed("button",self)
+		self.P2AjoyButton = QPushButton("button",self)
+		self.P1BjoyButton = QPushButton("button",self)
+		self.P2BjoyButton = QPushButton("button",self)
+		self.P1CjoyButton = QPushButton("button",self)
+		self.P2CjoyButton = QPushButton("button",self)
+		self.P1DjoyButton = QPushButton("button",self)
+		self.P2DjoyButton = QPushButton("button",self)
+		self.P1StartjoyButton = QPushButton("button",self)
+		self.P2StartjoyButton = QPushButton("button",self)
+		self.P1CoinjoyButton = QPushButton("button",self)
+		self.P2CoinjoyButton = QPushButton("button",self)
+		self.P1UpjoyButton = QPushButton("button",self)
+		self.P2UpjoyButton = QPushButton("button",self)
+		self.P1AjoyButton = QPushButton("button",self)
+		self.P1RightjoyButton = QPushButton("button",self)
+		self.P2RightjoyButton = QPushButton("button",self)
+		self.P2LeftjoyButton = QPushButton("button",self)
+		self.P1LeftjoyButton = QPushButton("button",self)
+		self.P2DownjoyButton = QPushButton("button",self)
+		self.P1DownjoyButton = QPushButton("button",self)
 		grid.addWidget(self.P1AjoyButton, 2, 2)
 		grid.addWidget(self.P2AjoyButton, 2, 4)
 		grid.addWidget(self.P2RightjoyButton, 11, 4)
@@ -1109,94 +1117,114 @@ class JoykeyWindow(QDialog):
 
 	self.setLayout(grid)
 	# P1
-	self.connect(self.P1AjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
-	self.connect(self.P1BjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
-	self.connect(self.P1CjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
-	self.connect(self.P1DjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
-	self.connect(self.P1StartjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
-	self.connect(self.P1CoinjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
-	self.connect(self.P1UpjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
-	self.connect(self.P1DownjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
-	self.connect(self.P1LeftjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
-	self.connect(self.P1RightjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
+	self.connect(self.P1AjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
+	self.connect(self.P1BjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
+	self.connect(self.P1CjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
+	self.connect(self.P1DjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
+	self.connect(self.P1StartjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
+	self.connect(self.P1CoinjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
+	self.connect(self.P1UpjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
+	self.connect(self.P1DownjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
+	self.connect(self.P1LeftjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
+	self.connect(self.P1RightjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
 	if (joystickAttached):
-		self.connect(self.P1RightjoyButton, SIGNAL("joykeyPressed"),self.P1joyupdateRight)
-		self.connect(self.P1LeftjoyButton, SIGNAL("joykeyPressed"),self.P1updateLeft)
-		self.connect(self.P1DownjoyButton, SIGNAL("joykeyPressed"),self.P1updateDown)
-		self.connect(self.P1UpjoyButton, SIGNAL("joykeyPressed"),self.P1updateUp)
-		self.connect(self.P1CoinjoyButton, SIGNAL("joykeyPressed"),self.P1updateCoin)
-		self.connect(self.P1StartjoyButton, SIGNAL("joykeyPressed"),self.P1updateStart)
-		self.connect(self.P1AjoyButton, SIGNAL("joykeyPressed"),self.P1updateA)
-		self.connect(self.P1BjoyButton, SIGNAL("joykeyPressed"),self.P1updateB)
-		self.connect(self.P1CjoyButton, SIGNAL("joykeyPressed"),self.P1updateC)
-		self.connect(self.P1DjoyButton, SIGNAL("joykeyPressed"),self.P1updateD)
+		self.connect(self.P1RightjoyButton, SIGNAL("clicked()"),self.P1joyupdateRight)
+		self.connect(self.P1LeftjoyButton, SIGNAL("clicked()"),self.P1updateLeft)
+		self.connect(self.P1DownjoyButton, SIGNAL("clicked()"),self.P1updateDown)
+		self.connect(self.P1UpjoyButton, SIGNAL("clicked()"),self.P1updateUp)
+		self.connect(self.P1CoinjoyButton, SIGNAL("clicked()"),self.P1updateCoin)
+		self.connect(self.P1StartjoyButton, SIGNAL("clicked()"),self.P1updateStart)
+		self.connect(self.P1AjoyButton, SIGNAL("clicked()"),self.P1updateA)
+		self.connect(self.P1BjoyButton, SIGNAL("clicked()"),self.P1updateB)
+		self.connect(self.P1CjoyButton, SIGNAL("clicked()"),self.P1updateC)
+		self.connect(self.P1DjoyButton, SIGNAL("clicked()"),self.P1updateD)
 	# P2
-	self.connect(self.P2AjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
-	self.connect(self.P2BjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
-	self.connect(self.P2CjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
-	self.connect(self.P2DjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
-	self.connect(self.P2StartjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
-	self.connect(self.P2CoinjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
-	self.connect(self.P2UpjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
-	self.connect(self.P2DownjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
-	self.connect(self.P2LeftjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
-	self.connect(self.P2RightjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value)
+	self.connect(self.P2AjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
+	self.connect(self.P2BjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
+	self.connect(self.P2CjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
+	self.connect(self.P2DjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
+	self.connect(self.P2StartjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
+	self.connect(self.P2CoinjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
+	self.connect(self.P2UpjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
+	self.connect(self.P2DownjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
+	self.connect(self.P2LeftjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
+	self.connect(self.P2RightjoykeySpinBox, SIGNAL('valueChanged(double)'), self.change_value_joy)
 	if (joystickAttached):
-		self.connect(self.P2RightjoyButton, SIGNAL("joykeyPressed"),self.P2updateRight)
-		self.connect(self.P2LeftjoyButton, SIGNAL("joykeyPressed"),self.P2updateLeft)
-		self.connect(self.P2DownjoyButton, SIGNAL("joykeyPressed"),self.P2updateDown)
-		self.connect(self.P2UpjoyButton, SIGNAL("joykeyPressed"),self.P2updateUp)
-		self.connect(self.P2CoinjoyButton, SIGNAL("joykeyPressed"),self.P2updateCoin)
-		self.connect(self.P2StartjoyButton, SIGNAL("joykeyPressed"),self.P2updateStart)
-		self.connect(self.P2AjoyButton, SIGNAL("joykeyPressed"),self.P2updateA)
-		self.connect(self.P2BjoyButton, SIGNAL("joykeyPressed"),self.P2updateB)
-		self.connect(self.P2CjoyButton, SIGNAL("joykeyPressed"),self.P2updateC)
-		self.connect(self.P2DjoyButton, SIGNAL("joykeyPressed"),self.P2updateD)
+		self.connect(self.P2RightjoyButton, SIGNAL("clicked()"),self.P2updateRight)
+		self.connect(self.P2LeftjoyButton, SIGNAL("clicked()"),self.P2updateLeft)
+		self.connect(self.P2DownjoyButton, SIGNAL("clicked()"),self.P2updateDown)
+		self.connect(self.P2UpjoyButton, SIGNAL("clicked()"),self.P2updateUp)
+		self.connect(self.P2CoinjoyButton, SIGNAL("clicked()"),self.P2updateCoin)
+		self.connect(self.P2StartjoyButton, SIGNAL("clicked()"),self.P2updateStart)
+		self.connect(self.P2AjoyButton, SIGNAL("clicked()"),self.P2updateA)
+		self.connect(self.P2BjoyButton, SIGNAL("clicked()"),self.P2updateB)
+		self.connect(self.P2CjoyButton, SIGNAL("clicked()"),self.P2updateC)
+		self.connect(self.P2DjoyButton, SIGNAL("clicked()"),self.P2updateD)
 
 	# P1
     def P1joyupdateRight(self):
+	MyJoykeyPressed()
 	self.P1RightjoykeySpinBox.setValue(float(joyVal))
     def P1updateLeft(self):
+	MyJoykeyPressed()
 	self.P1LeftjoykeySpinBox.setValue(float(joyVal))
     def P1updateUp(self):
+	MyJoykeyPressed()
 	self.P1UpjoykeySpinBox.setValue(float(joyVal))
     def P1updateDown(self):
+	MyJoykeyPressed()
 	self.P1DownjoykeySpinBox.setValue(float(joyVal))
     def P1updateStart(self):
+	MyJoykeyPressed()
 	self.P1StartjoykeySpinBox.setValue(float(joyVal))
     def P1updateCoin(self):
+	MyJoykeyPressed()
 	self.P1CoinjoykeySpinBox.setValue(float(joyVal))
     def P1updateA(self):
+	MyJoykeyPressed()
 	self.P1AjoykeySpinBox.setValue(float(joyVal))
     def P1updateB(self):
+	MyJoykeyPressed()
 	self.P1BjoykeySpinBox.setValue(float(joyVal))
     def P1updateC(self):
+	MyJoykeyPressed()
 	self.P1CjoykeySpinBox.setValue(float(joyVal))
     def P1updateD(self):
+	MyJoykeyPressed()
 	self.P1DjoykeySpinBox.setValue(float(joyVal))
 	# P2
     def P2updateRight(self):
+	MyJoykeyPressed()
 	self.P2RightjoykeySpinBox.setValue(float(joyVal))
     def P2updateLeft(self):
+	MyJoykeyPressed()
 	self.P2LeftjoykeySpinBox.setValue(float(joyVal))
     def P2updateUp(self):
+	MyJoykeyPressed()
 	self.P2UpjoykeySpinBox.setValue(float(joyVal))
     def P2updateDown(self):
+	MyJoykeyPressed()
 	self.P2DownjoykeySpinBox.setValue(float(joyVal))
     def P2updateStart(self):
+	MyJoykeyPressed()
 	self.P2StartjoykeySpinBox.setValue(float(joyVal))
     def P2updateCoin(self):
+	MyJoykeyPressed()
 	self.P2CoinjoykeySpinBox.setValue(float(joyVal))
     def P2updateA(self):
+	MyJoykeyPressed()
 	self.P2AjoykeySpinBox.setValue(float(joyVal))
     def P2updateB(self):
+	MyJoykeyPressed()
 	self.P2BjoykeySpinBox.setValue(float(joyVal))
     def P2updateC(self):
+	MyJoykeyPressed()
 	self.P2CjoykeySpinBox.setValue(float(joyVal))
     def P2updateD(self):
+	MyJoykeyPressed()
 	self.P2DjoykeySpinBox.setValue(float(joyVal))
 
-    def change_value(self,joyevent):
+    def change_value_joy(self):
 	# P1
 	self.P1joyKeys[0] = "%.0f" % self.P1AjoykeySpinBox.value()
 	self.P1joyKeys[1] = "%.0f" % self.P1BjoykeySpinBox.value()
@@ -1266,47 +1294,51 @@ class MyKeyPressed(QPushButton):
         return QPushButton.event(self, event)
 
 class MyJoykeyPressed(QPushButton):
-    def __init__(self, *args):
-        QPushButton.__init__(self, *args)
-	#sys.exit()
+    def __init__(self, parent=None):
+        QPushButton.__init__(self, parent)
 
-    def joyevent(self,joyevent):
 	global joyVal
-	joyVal = 1
-	#pipe = open('/dev/input/js0','r')
+	pipe = open('/dev/input/js0','r')
 	action = []
 	stopping = False
 	while not stopping :
 		for character in pipe.read(1):
 			action += ['%02X' % ord(character)]
 			if len(action) == 8:
-
+	
 				if action[6] == '01': # Button
 					if action[4] == '01':
 						joyVal = action[7]
 						stopping = True
+						break
 				elif action[7] == '04': # Left Joystick left/right
 					if action[4] == 'FF':
-						print 'You pressed right on the left joystick [4,1]'
+						# You pressed right on the left joystick
 						joyVal = 4 #,1
 						stopping = True
 					elif action[4] == '01':
-						print 'You pressed left on the left joystick [4,1]'
+						# You pressed left on the left joystick
 						joyVal = 4 #,1
 						stopping = True
 				elif action[7] == '05': # Left Joystick up/down
 					if action[4] == 'FF':
-						print 'You pressed down on the left joystick [5,1]'
+						# You pressed down on the left joystick
 						joyVal = 5 #,1
 						stopping = True
 					elif action[4] == '01':
-						joyVal = 5 #,1
-						print 'You pressed up on the left joystick [5,1]'
+						joyVal = 4 #,1
+						# You pressed up on the left joystick 
 						stopping = True
 
 				action = []
-	self.emit(SIGNAL("joykeyPressed"))
-        return QPushButton.event(self, joyevent)
+				if stopping:
+					break
+			if stopping:
+				break
+		if stopping:
+			break
+	pipe.close()
+
  
 class pathWindow(QDialog):
     def __init__(self, parent=None):
